import { Observable } from "typescript-observable";
import { ICart } from "./interfaces/cart";
import { IWeightUnit } from "./interfaces/weight-unit";
import { ICurrency } from "./interfaces/currency";
import { IProduct } from "./interfaces/product";
import { ICartConfig, ICartOptionalConfig } from "./interfaces/cart-config";
import { ICurrencyConverter } from "./interfaces/currency-converter";
import { IAddProduct } from "./interfaces/product-data";
export declare class Cart extends Observable implements ICart {
    private config;
    private currencyConverter;
    private currency;
    private weightUnit;
    private content;
    constructor(config?: ICartOptionalConfig, currencyConverter?: ICurrencyConverter);
    /**
     * Get the cart config.
     * @returns {ICartConfig} The cart config.
     */
    getConfig(): ICartConfig;
    /**
     * Get the number of items in the cart.
     * @returns {number} The number of products in the cart.
     */
    count(): number;
    /**
     * Get the products in the cart.
     * @return {IProduct[]} The products in the cart.
     */
    getContents(): IProduct[];
    get(id: number): IProduct | null;
    /**
     * Add a single product to the cart
     * @param {IAddProduct} product The product to add
     * @returns {IProduct} The added product
     */
    addItem(product: IAddProduct): IProduct;
    /**
     * Add multiple products
     * @param {IAddProduct[]} product The products to add in a list
     * @returns {IProduct[]} The added products in a list
     */
    addItem(product: IAddProduct[]): IProduct[];
    /**
     * Search for a product in the cart given an IAddProduct.
     * @param {IAddProduct} product The product to find in the cart.
     * @returns {number} The index of the product if it exist, otherwise -1.
     */
    private findIndex(product);
    private static isAddProduct(product);
    /**
     * Update the quantity of a product in the basket.
     * @param {number} id
     * @param {number} quantity
     * @return {boolean}
     */
    updateItemQuantity(id: number, quantity: number): boolean;
    /**
     * Remove an item from the cart
     * @param {number} id is the id of the product
     * @return {boolean} true if successful, otherwise false
     */
    removeItem(id: number): boolean;
    /**
     * Clear the cart from all items.
     * @returns {ICart} Chaining this
     */
    clear(): ICart;
    /**
     * Set the currency converter of this
     * @param {ICurrencyConverter} currencyConverter
     * @returns {ICart} Chaining this
     */
    setCurrencyConverter(currencyConverter: ICurrencyConverter): ICart;
    /**
     * Get the total price of the cart. The price is given in the currency set in the cart.
     * @param {boolean} vat If VAT should be included in the price.
     * @returns {number}
     */
    getTotalPrice(vat?: boolean): number | null;
    /**
     * Get the total VAT of the products in the cart. Given in the currency set in the cart.
     * @returns {number} The VAT of all products.
     */
    getVat(): number | null;
    /**
     * The currency of the cart.
     * @returns {ICurrency} The currency of the cart.
     */
    getCurrency(): ICurrency;
    /**
     * Set the currency of the cart.
     * @param {ICurrency} currency The new currency to use in the cart.
     * @returns {ICart} Chained this
     */
    setCurrency(currency: ICurrency): ICart;
    /**
     * Get the default currency of the cart. This is the currency the cart expects new items to be given in.
     * @returns {ICurrency} The default currency of the cart.
     */
    getDefaultCurrency(): ICurrency;
    /**
     * Get the weight of the cart.
     * @returns {number}
     */
    getWeight(): number;
    /**
     * Get the weight unit system for this cart.
     * @returns {IWeightUnit} The weight unit system (e.g. kilogram).
     */
    getWeightUnitSystem(): IWeightUnit;
    /**
     * Set the weight unit system for this cart.
     * @param {IWeightUnit} weightUnit The new weight unit system.
     * @returns {ICart} Chaining this
     */
    setWeightUnitSystem(weightUnit: IWeightUnit): ICart;
    /**
     * Get the default unit system of this cart. This is the unit system the cart expects new products to be given in.
     * @returns {IWeightUnit}
     */
    getDefaultUnitSystem(): IWeightUnit;
    /**
     * Export the cart to a JSON string.
     * @returns {string} The JSON string
     */
    exportCart(): string;
    /**
     * Import the cart from a JSON string.
     * @param {string} data The JSON string to import.
     * @returns {ICart | null} The imported cart if the import was successful. Otherwise null.
     */
    static importCart(data: string): ICart | null;
    private static isExportedCart(obj);
}
