"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var typescript_observable_1 = require("typescript-observable");
var currencies_1 = require("./currencies/currencies");
var weight_units_1 = require("./weight-units/weight-units");
var product_1 = require("./product");
var cart_events_1 = require("./cart-events");
var product_events_1 = require("./product-events");
var currency_converter_1 = require("./currency-converter");
var deepEqual = require("deep-equal");
var Cart = /** @class */ (function (_super) {
    __extends(Cart, _super);
    function Cart(config, currencyConverter) {
        var _this = _super.call(this) || this;
        _this.content = [];
        // Set config object
        config = config || {};
        _this.config = {
            defaultCurrency: config.defaultCurrency || currencies_1.Currencies.EUR,
            defaultWeightUnit: config.defaultWeightUnit || weight_units_1.WeightUnits.KILOGRAM,
            defaultVat: config.defaultVat || 0.25,
            vatInPrice: config.vatInPrice !== false,
            stackAddedProducts: config.stackAddedProducts !== false
        };
        _this.currencyConverter = currencyConverter || new currency_converter_1.CurrencyConverter({});
        _this.currency = _this.config.defaultCurrency;
        _this.weightUnit = _this.config.defaultWeightUnit;
        return _this;
    }
    /**
     * Get the cart config.
     * @returns {ICartConfig} The cart config.
     */
    Cart.prototype.getConfig = function () {
        return this.config;
    };
    /**
     * Get the number of items in the cart.
     * @returns {number} The number of products in the cart.
     */
    Cart.prototype.count = function () {
        return this.content
            .map(function (item) { return item.getQuantity(); })
            .reduce(function (a, b) { return a + b; }, 0);
    };
    ;
    /**
     * Get the products in the cart.
     * @return {IProduct[]} The products in the cart.
     */
    Cart.prototype.getContents = function () {
        return this.content;
    };
    Cart.prototype.get = function (id) {
        var index = this.content.map(function (product) { return product.getId(); }).indexOf(id);
        if (index !== -1) {
            return this.content[index];
        }
        return null;
    };
    Cart.prototype.addItem = function (product) {
        var _this = this;
        // If multiple products is added, then add the one by one
        if (!Cart.isAddProduct(product)) {
            var products_1 = [];
            product.forEach(function (item) {
                products_1.push(_this.addItem(item));
            });
            return products_1;
        }
        // Add possible missing attributes (needed to find the product)
        product.name = product.name || '';
        product.quantity = product.quantity || 1;
        product.weight = product.weight || 0;
        product.additionPrice = product.additionPrice || 0;
        product.vat = product.vat || this.getConfig().defaultVat;
        product.extra = product.extra || {};
        // If the products should be stacked then try to add it to a previous item
        if (this.config.stackAddedProducts) {
            // Get the index. If it is found then add it to the existing product.
            var index = this.findIndex(product);
            if (index !== -1) {
                // Update the quantity
                this.content[index].setQuantity(product.quantity, true);
                // Notify observers
                this.notify(cart_events_1.default.PRODUCT_QUANTITY_CHANGED, {
                    product: this.content[index]
                });
                return this.content[index];
            }
        }
        // Add a completely new product.
        var newProduct = new product_1.Product(this.content.length, product, this.getConfig());
        this.content.push(newProduct);
        newProduct.on(product_events_1.default.REQUEST_DELETE, function (data) {
            _this.removeItem(data.id);
        });
        // Notify observers
        this.notify(cart_events_1.default.PRODUCT_ADD, {
            product: newProduct
        });
        return newProduct;
    };
    /**
     * Search for a product in the cart given an IAddProduct.
     * @param {IAddProduct} product The product to find in the cart.
     * @returns {number} The index of the product if it exist, otherwise -1.
     */
    Cart.prototype.findIndex = function (product) {
        var index = -1;
        this.content.some(function (item, itemIndex) {
            var data = item.getData();
            if (data.sku === product.sku && data.basePrice === product.basePrice
                && data.additionPrice === product.additionPrice && data.weight === product.weight
                && data.vat === product.vat && deepEqual(data.extra, product.extra)) {
                index = itemIndex;
                return true;
            }
            return false;
        });
        return index;
    };
    Cart.isAddProduct = function (product) {
        return product.hasOwnProperty('sku');
    };
    /**
     * Update the quantity of a product in the basket.
     * @param {number} id
     * @param {number} quantity
     * @return {boolean}
     */
    Cart.prototype.updateItemQuantity = function (id, quantity) {
        var index = this.content.map(function (product) { return product.getId(); }).indexOf(id);
        if (index !== -1) {
            this.content[index].setQuantity(quantity);
            this.notify(cart_events_1.default.PRODUCT_QUANTITY_CHANGED, {
                product: this.content[index]
            });
            return true;
        }
        return false;
    };
    /**
     * Remove an item from the cart
     * @param {number} id is the id of the product
     * @return {boolean} true if successful, otherwise false
     */
    Cart.prototype.removeItem = function (id) {
        var index = this.content.map(function (product) { return product.getId(); }).indexOf(id);
        if (index !== -1) {
            var deletedProduct = this.content[index];
            this.content.splice(index, 1);
            // Notify observers
            this.notify(cart_events_1.default.PRODUCT_REMOVE, {
                product: deletedProduct
            });
            return true;
        }
        return false;
    };
    /**
     * Clear the cart from all items.
     * @returns {ICart} Chaining this
     */
    Cart.prototype.clear = function () {
        this.content.splice(0, this.content.length);
        this.notify(cart_events_1.default.CART_CLEAR, {});
        return this;
    };
    /**
     * Set the currency converter of this
     * @param {ICurrencyConverter} currencyConverter
     * @returns {ICart} Chaining this
     */
    Cart.prototype.setCurrencyConverter = function (currencyConverter) {
        this.currencyConverter = currencyConverter;
        return this;
    };
    /**
     * Get the total price of the cart. The price is given in the currency set in the cart.
     * @param {boolean} vat If VAT should be included in the price.
     * @returns {number}
     */
    Cart.prototype.getTotalPrice = function (vat) {
        if (vat === void 0) { vat = true; }
        return this.currencyConverter.convert({
            amount: this.content.map(function (product) { return product.getTotalPrice(vat); }).reduce(function (a, b) { return a + b; }, 0),
            from: this.getDefaultCurrency(),
            to: this.getCurrency()
        });
    };
    /**
     * Get the total VAT of the products in the cart. Given in the currency set in the cart.
     * @returns {number} The VAT of all products.
     */
    Cart.prototype.getVat = function () {
        return this.currencyConverter.convert({
            amount: this.content.map(function (product) { return product.getVat(); }).reduce(function (a, b) { return a + b; }, 0),
            from: this.getDefaultCurrency(),
            to: this.getCurrency()
        });
    };
    /**
     * The currency of the cart.
     * @returns {ICurrency} The currency of the cart.
     */
    Cart.prototype.getCurrency = function () {
        return this.currency;
    };
    /**
     * Set the currency of the cart.
     * @param {ICurrency} currency The new currency to use in the cart.
     * @returns {ICart} Chained this
     */
    Cart.prototype.setCurrency = function (currency) {
        this.currency = currency;
        return this;
    };
    /**
     * Get the default currency of the cart. This is the currency the cart expects new items to be given in.
     * @returns {ICurrency} The default currency of the cart.
     */
    Cart.prototype.getDefaultCurrency = function () {
        return this.config.defaultCurrency;
    };
    /**
     * Get the weight of the cart.
     * @returns {number}
     */
    Cart.prototype.getWeight = function () {
        var _this = this;
        return this.content
            .map(function (product) { return product.getWeight(_this.weightUnit); })
            .reduce(function (a, b) { return a + b; }, 0);
    };
    /**
     * Get the weight unit system for this cart.
     * @returns {IWeightUnit} The weight unit system (e.g. kilogram).
     */
    Cart.prototype.getWeightUnitSystem = function () {
        return this.weightUnit;
    };
    /**
     * Set the weight unit system for this cart.
     * @param {IWeightUnit} weightUnit The new weight unit system.
     * @returns {ICart} Chaining this
     */
    Cart.prototype.setWeightUnitSystem = function (weightUnit) {
        this.weightUnit = weightUnit;
        return this;
    };
    /**
     * Get the default unit system of this cart. This is the unit system the cart expects new products to be given in.
     * @returns {IWeightUnit}
     */
    Cart.prototype.getDefaultUnitSystem = function () {
        return this.config.defaultWeightUnit;
    };
    /**
     * Export the cart to a JSON string.
     * @returns {string} The JSON string
     */
    Cart.prototype.exportCart = function () {
        var exportedCart = {
            config: this.getConfig(),
            products: this.content.map(function (product) { return ({
                sku: product.getData().sku,
                name: product.getData().name,
                basePrice: product.getData().basePrice,
                vat: product.getData().vat,
                additionPrice: product.getData().additionPrice,
                quantity: product.getData().quantity,
                weight: product.getData().weight,
                extra: product.getData().extra
            }); })
        };
        return JSON.stringify(exportedCart);
    };
    /**
     * Import the cart from a JSON string.
     * @param {string} data The JSON string to import.
     * @returns {ICart | null} The imported cart if the import was successful. Otherwise null.
     */
    Cart.importCart = function (data) {
        var decodedJson = JSON.parse(data);
        if (Cart.isExportedCart(decodedJson)) {
            // Saved JSON can not store functions. Restore the correct currency class.
            decodedJson.config.defaultCurrency = currencies_1.Currencies.identifyCurrency(decodedJson.config.defaultCurrency);
            decodedJson.config.defaultWeightUnit = weight_units_1.WeightUnits.identifyWeightUnit(decodedJson.config.defaultWeightUnit);
            var cart = new Cart(decodedJson.config);
            cart.addItem(decodedJson.products);
            return cart;
        }
        return null;
    };
    Cart.isExportedCart = function (obj) {
        return obj.hasOwnProperty('config') && obj.hasOwnProperty('products');
    };
    return Cart;
}(typescript_observable_1.Observable));
exports.Cart = Cart;
//# sourceMappingURL=cart.js.map