"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var typescript_observable_1 = require("typescript-observable");
var product_events_1 = require("./product-events");
var Product = /** @class */ (function (_super) {
    __extends(Product, _super);
    function Product(id, data, cartConfig) {
        var _this = _super.call(this) || this;
        _this.id = id;
        // Set product data
        _this.productData = {
            sku: data.sku,
            name: data.name || '',
            quantity: data.quantity || 1,
            weightUnitSystem: cartConfig.defaultWeightUnit,
            weight: data.weight || 0,
            currency: cartConfig.defaultCurrency,
            basePrice: data.basePrice,
            additionPrice: data.additionPrice || 0,
            vat: data.vat || cartConfig.defaultVat,
            vatInPrice: cartConfig.vatInPrice,
            extra: data.extra || {}
        };
        return _this;
    }
    /**
     * Get the id of the product. This id is specific for the cart of the products.
     * @returns {number} The id
     */
    Product.prototype.getId = function () {
        return this.id;
    };
    /**
     * Get the name of the product
     * @returns {string} The product name
     */
    Product.prototype.getName = function () {
        return this.productData.name;
    };
    /**
     * Get the product data. This object contains all data about the object. Use functions in this class if possible to
     * change the object rather than changing it directly.
     * @returns {IProductData}
     */
    Product.prototype.getData = function () {
        return this.productData;
    };
    /**
     * Get the quantity of the product.
     * @returns {number} The quantity
     */
    Product.prototype.getQuantity = function () {
        return this.getData().quantity;
    };
    /**
     * Set the quantity.
     * @param {number} quantity The new quantity.
     * @param {boolean} add If true the quantity is added, if false the new quantity will overwrite the old.
     * @returns {IProduct} Chaining this
     */
    Product.prototype.setQuantity = function (quantity, add) {
        if (add === void 0) { add = false; }
        // Make sure that quantity is a number
        quantity = Number(quantity);
        if (add) {
            this.getData().quantity += quantity;
            return this;
        }
        this.getData().quantity = quantity;
        // Remove if the quantity is less or equal to zero
        if (this.getQuantity() <= 0) {
            this.remove();
        }
        return this;
    };
    /**
     * Remove the product from cart
     * @return {Promise<void>} A promise when the product is removed
     */
    Product.prototype.remove = function () {
        return this.notify(product_events_1.default.REQUEST_DELETE, {
            id: this.getId()
        });
    };
    /**
     * Get the weight of the product
     * @param {IWeightUnit} weightUnit the weight units to use
     * @returns {number} The weight of the cart
     */
    Product.prototype.getWeight = function (weightUnit) {
        // Convert the weight to kilograms
        var weightInKilogram = this.getData().weightUnitSystem.convertToKilogram(this.getData().weight || 0);
        // Convert the weight to the unit requested
        return weightUnit.convertFromKilogram(weightInKilogram) * this.getQuantity();
    };
    /**
     * Get the total price of this product
     * @param {boolean} vat
     * @returns {number}
     */
    Product.prototype.getTotalPrice = function (vat) {
        var totalPrice = this.getData().basePrice + this.getData().additionPrice;
        // If the price is already saved in the correct format
        if (this.getData().vatInPrice === vat) {
            return totalPrice * this.getQuantity();
        }
        else if (!this.getData().vatInPrice) {
            return totalPrice * (1 + this.getData().vat) * this.getQuantity();
        }
        // VAT should be removed from the price
        return totalPrice / (1 + this.getData().vat) * this.getQuantity();
    };
    /**
     * Get the currency of this product.
     * @returns {ICurrency} The currency
     */
    Product.prototype.getCurrency = function () {
        return this.getData().currency;
    };
    /**
     * Get the vat of this product
     * @returns {number} The vat
     */
    Product.prototype.getVat = function () {
        var totalPrice = this.getData().basePrice + this.getData().additionPrice;
        // VAT is in the total price
        if (this.getData().vatInPrice) {
            return this.getData().vat * totalPrice / (1 + this.getData().vat) * this.getQuantity();
        }
        // VAT is not in the price
        return totalPrice * this.getData().vat * this.getQuantity();
    };
    /**
     * Get the extra data of this product
     * @returns {any}
     */
    Product.prototype.getExtra = function () {
        return this.getData().extra;
    };
    return Product;
}(typescript_observable_1.Observable));
exports.Product = Product;
//# sourceMappingURL=product.js.map